
"""
Standard.py

This module defines standard chemical properties and parameters used throughout the Vorpy package.
It contains essential data structures for atomic radii and specialized radii for different amino acids.

The module provides:
- element_radii: Dictionary mapping element symbols to their van der Waals radii
- special_radii: Dictionary mapping amino acid names to dictionaries of atom-specific radii

The radii values are used for:
- Molecular surface calculations
- Volume calculations
- Distance-based analyses
- Molecular visualization

Note: All radii values are in Angstroms (Å)
"""


element_radii = {
    'H': 1.30, 'HE': 1.40, 'LI': 0.76, 'BE': 0.45, 'B': 1.92, 'C': 1.80, 'N': 1.60, 'O': 1.50, 'F': 1.33,
    'NE': 1.54, 'NA': 1.02, 'MG': 0.72, 'AL': 0.60, 'SI': 2.10, 'P': 1.90, 'S': 1.90, 'CL': 1.81, 'AR': 1.88,
    'K': 1.38, 'CA': 1.00, 'GA': 0.62, 'GE': 0.73, 'AS': 0.58, 'SE': 1.90, 'BR': 1.83, 'KR': 2.02, 'RB': 1.52,
    'SR': 1.18, 'IN': 1.93, 'SN': 2.17, 'SB': 2.06, 'TE': 2.06, 'I': 2.20, 'XE': 2.16, 'CS': 1.67, 'BA': 1.35,
    'TL': 1.96, 'PB': 2.02, 'BI': 2.07, 'PO': 1.97, 'AT': 2.02, 'RN': 2.20, 'FR': 3.48, 'RA': 2.83, 'ZN': 1.39
}

special_radii = {
    'ALA': {
        'C': 1.8, 'CA': 1.8, 'CB': 1.92, 'H': 1.3, 'HA': 1.3, 'HB1': 1.3, 'HB2': 1.3, 'HB3': 1.3, 'N': 1.6, 'O': 1.5, 'OC1': 1.5, 'OC2': 1.5
    },
    'ARG': {
        'C': 1.8, 'CA': 1.8, 'CB': 1.91, 'CD': 1.88, 'CG': 1.92, 'CZ': 1.8, 'H': 1.3, 'HA': 1.3, 'HB1': 1.3, 'HB2': 1.3, 'HB3': 1.3, 'HD1': 1.3, 'HD2': 1.3, 'HD3': 1.3, 'HE': 1.3, 'HG1': 1.3, 'HG2': 1.3, 'HH11': 1.3, 'HH12': 1.3, 'HH21': 1.3, 'HH22': 1.3, 'N': 1.6, 'NE': 1.62, 'NH1': 1.62, 'NH2': 1.67, 'O': 1.5
    },
    'ASN': {
        'C': 1.8, 'CA': 1.8, 'CB': 1.91, 'CG': 1.81, 'H': 1.3, 'HA': 1.3, 'HB1': 1.3, 'HB2': 1.3, 'HD21': 1.3, 'HD22': 1.3, 'N': 1.6, 'ND2': 1.62, 'O': 1.5, 'OD1': 1.52
    },
    'ASP': {
        'C': 1.8, 'CA': 1.8, 'CB': 1.91, 'CG': 1.76, 'H': 1.3, 'HA': 1.3, 'HB1': 1.3, 'HB2': 1.3, 'N': 1.6, 'O': 1.5, 'OD1': 1.49, 'OD2': 1.49
    },
    'CYS': {
        'C': 1.8, 'CA': 1.8, 'CB': 1.91, 'H': 1.3, 'HA': 1.3, 'HB1': 1.3, 'HB2': 1.3, 'HG': 1.3, 'N': 1.6, 'O': 1.5, 'S': 1.9, 'SG': 1.9
    },
    'GLN': {
        'C': 1.8, 'CA': 1.8, 'CB': 1.91, 'CD': 1.81, 'CG': 1.8, 'H': 1.3, 'HA': 1.3, 'HB1': 1.3, 'HB2': 1.3, 'HE21': 1.62, 'HE22': 1.62, 'HG1': 1.3, 'HG2': 1.3, 'N': 1.6, 'NE2': 1.62, 'O': 1.5, 'OE1': 1.52
    },
    'GLU': {
        'C': 1.8, 'CA': 1.8, 'CB': 1.91, 'CD': 1.76, 'CG': 1.88, 'H': 1.3, 'HA': 1.3, 'HB1': 1.3, 'HB2': 1.3, 'HG1': 1.3, 'HG2': 1.3, 'N': 1.6, 'O': 1.5, 'OE1': 1.49, 'OE2': 1.49
    },
    'GLY': {
        'C': 1.8, 'CA': 1.8, 'H': 1.3, 'HA1': 1.3, 'HA2': 1.3, 'N': 1.6, 'O': 1.5, 'OC1': 1.5, 'OC2': 1.5
    },
    'HIS': {
        'C': 1.8, 'CA': 1.8, 'CB': 1.91, 'CD': 1.74, 'CE': 1.74, 'CD2': 1.74, 'CE1': 1.74, 'CG': 1.8, 'H': 1.3, 'HA': 1.3, 'HB1': 1.3, 'HB2': 1.3, 'HD2': 1.3, 'HE1': 1.3, 'HE2': 1.3, 'N': 1.6, 'ND1': 1.6, 'ND2': 1.6, 'NE2': 1.6, 'O': 1.5
    },
    'ILE': {
        'C': 1.8, 'CA': 1.8, 'CB': 2.01, 'CD': 1.92, 'CD1': 1.92, 'CG1': 1.92, 'CG2': 1.92, 'H': 1.3, 'HA': 1.3, 'HB': 1.3, 'HD1': 1.3, 'HD2': 1.3, 'HD3': 1.3, 'HD11': 1.3, 'HD12': 1.3, 'HD13': 1.3, 'HG12': 1.3, 'HG13': 1.3, 'HG21': 1.3, 'HG22': 1.3, 'HG23': 1.3, 'N': 1.6, 'O': 1.5
    },
    'LEU': {
        'C': 1.8, 'CA': 1.8, 'CB': 1.91, 'CD1': 1.92, 'CD2': 1.92, 'CG': 2.01, 'H': 1.3, 'HA': 1.3, 'HB1': 1.3, 'HB2': 1.3, 'HD11': 1.3, 'HD12': 1.3, 'HD13': 1.3, 'HD21': 1.3, 'HD22': 1.3, 'HD23': 1.3, 'HG': 1.3, 'N': 1.6, 'O': 1.5
    },
    'LYS': {
        'C': 1.8, 'CA': 1.8, 'CB': 1.91, 'CD': 1.92, 'CE': 1.88, 'CG': 1.92, 'H': 1.3, 'HA': 1.3, 'HB1': 1.3, 'HB2': 1.3, 'HD1': 1.3, 'HD2': 1.3, 'HE1': 1.3, 'HE2': 1.3, 'HG1': 1.3, 'HG2': 1.3, 'HZ1': 1.67, 'HZ2': 1.67, 'HZ3': 1.67, 'N': 1.6, 'NZ': 1.67, 'O': 1.5
    },
    'MET': {
        'C': 1.8, 'CA': 1.8, 'CB': 1.91, 'CE': 1.8, 'CG': 1.92, 'H': 1.3, 'HA': 1.3, 'HB1': 1.3, 'HB2': 1.3, 'HE1': 1.3, 'HE2': 1.3, 'HE3': 1.3, 'HG1': 1.3, 'HG2': 1.3, 'N': 1.6, 'O': 1.5, 'SD': 1.94, 'S': 1.94
    },
    'PHE': {
        'C': 1.8, 'CA': 1.8, 'CB': 1.91, 'CD1': 1.82, 'CD2': 1.82, 'CD': 1.82, 'CE1': 1.82, 'CE2': 1.82, 'CG': 1.74, 'CZ': 1.82, 'H': 1.3, 'HA': 1.3, 'HB1': 1.3, 'HB2': 1.3, 'HD1': 1.3, 'HD2': 1.3, 'HE1': 1.3, 'HE2': 1.3, 'HZ': 1.3, 'N': 1.6, 'O': 1.5
    },
    'PRO': {
        'C': 1.8, 'CA': 1.8, 'CB': 1.91, 'CD': 1.92, 'CG': 1.92, 'H': 1.3, 'HA': 1.3, 'HB1': 1.3, 'HB2': 1.3, 'HD1': 1.3, 'HD2': 1.3, 'HG1': 1.3, 'HG2': 1.3, 'N': 1.6, 'O': 1.5
    },
    'SER': {
        'C': 1.8, 'CA': 1.8, 'CB': 1.91, 'H1': 1.3, 'H2': 1.3, 'H3': 1.3, 'H': 1.3, 'HA': 1.3, 'HB1': 1.3, 'HB2': 1.3, 'HG': 1.54, 'N': 1.6, 'O': 1.5, 'OG': 1.54
    },
    'THR': {
        'C': 1.8, 'CA': 1.8, 'CB': 2.01, 'CG2': 1.92, 'H': 1.3, 'HA': 1.3, 'HB': 1.3, 'HG1': 1.54, 'HG21': 1.3, 'HG22': 1.3, 'HG23': 1.3, 'N': 1.6, 'O': 1.5, 'OG1': 1.54
    },
    'TRP': {
        'C': 1.8, 'CA': 1.8, 'CB': 1.91, 'CD1': 1.82, 'CD2': 1.82, 'CD': 1.82, 'CE': 1.82, 'CE2': 1.74, 'CE3': 1.74, 'CG': 1.74, 'CH': 1.82, 'CH2': 1.82, 'CZ': 1.82, 'CZ1': 1.82, 'CZ2': 1.82, 'H': 1.3, 'HA': 1.3, 'HB1': 1.3, 'HB2': 1.3, 'HD1': 1.3, 'HE1': 1.62, 'HE3': 1.3, 'HZ1': 1.3, 'HZ2': 1.3, 'HH2': 1.3, 'N': 1.6, 'NE1': 1.66, 'O': 1.5
    },
    'TYR': {
        'C': 1.8, 'CA': 1.8, 'CB': 1.91, 'CD': 1.82, 'CD1': 1.82, 'CD2': 1.82, 'CE': 1.82, 'CE1': 1.82, 'CE2': 1.82, 'CG': 1.74, 'CZ': 1.8, 'H': 1.3, 'HA': 1.3, 'HB1': 1.3, 'HB2': 1.3, 'HD1': 1.3, 'HD2': 1.3, 'HE1': 1.3, 'HE2': 1.3, 'HH': 1.3, 'N': 1.6, 'O': 1.5, 'OH': 1.54
    },
    'VAL': {
        'C': 1.8, 'CA': 1.8, 'CB': 2.01, 'CG1': 1.92, 'CG2': 1.92, 'H': 1.3, 'HA': 1.3, 'HB': 1.3, 'HG11': 1.3, 'HG12': 1.3, 'HG13': 1.3, 'HG21': 1.3, 'HG22': 1.3, 'HG23': 1.3, 'N': 1.6, 'O': 1.5
    }
}
